"""checkpasswords: Uses pass_import to read a password manager source file storing raw
data and infers data such as:

- zxcvbnScore
- isPasswordDuplicate
- passwordPrint
- isHttp
- isMfaAvailable
- isMfaEnabled

Used to:

- check for duplicate passwords
- check for weak passwords
- identify http sites
- list available 2fa options using data from https://2fa.directory/
- list emails to submit to HIBP or similar
"""

import argparse
from sys import stdout

from pass_import.__main__ import MANAGERS

from checkpasswords.io import input as input_
from checkpasswords.io import output


def main():
	"""Main entry point."""
	outputMap = {
		"ansi": output.ansi,
		"plain": output.plainText,
		"markdown": output.markdown,
		"json": output.jsonF,
		"raw": output.raw,
		"raw-csv": output.rawCsv,
	}
	parser = argparse.ArgumentParser(
		description=__doc__, formatter_class=argparse.RawTextHelpFormatter
	)
	parser.add_argument(
		"credentials",
		help="Credentials/ passwords file to check",
	)
	parser.add_argument(
		"--output-format",
		"-o",
		help=f"""
Output format. One of {list(outputMap)}. default=ansi""".replace(
			"\n", ""
		),
	)
	parser.add_argument(
		"--input-format",
		"-i",
		help=f"""
Input format. One of {MANAGERS.names()}""".replace(
			"\n", ""
		),
	)
	parser.add_argument(
		"--file",
		"-f",
		help="Filename to write to (omit for stdout)",
	)
	parser.add_argument(
		"--no-colour",
		"-z",
		help="No ANSI colours",
		action="store_true",
	)

	args = parser.parse_args()
	# File
	filename = (
		stdout
		if args.file is None
		else open(args.file, "w", encoding="utf-8")  # pylint: disable=consider-using-with
	)

	outputFunction = outputMap.get(args.output_format, output.ansi)
	# no_colour
	if outputFunction == output.ansi and args.no_colour:  # pylint: disable=comparison-with-callable
		outputFunction = output.plainText

	print(
		outputFunction(input_.passImport(args.credentials, args.input_format)),
		file=filename,
	)


if __name__ == "__main__":
	main()



"""Output

- summary table
- duplicate passwords table
- weak passwords table
- http sites table
- enable2fa table
- emails table

To one of the following formats:

- ansi
- plain
- markdown
- json
- raw
- raw-csv
"""

import csv
import json
import re
from io import StringIO

from rich.console import Console
from rich.table import Table

from checkpasswords.credentials import Credentials, generateTables

INFO = {"program": "checkpasswords", "version": "2022"}


def stripAnsi(string: str) -> str:
	"""Strip ansi codes from a given string

	Args:
		string (str): string to strip codes from

	Returns:
		str: plaintext, utf-8 string (safe for writing to files)
	"""
	return re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])").sub("", string)


def ansi(credentials: list[Credentials]) -> str:
	"""Format to ansi

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in ansi format
	"""
	(
		duplicatePasswordsTable,
		weakPasswordsTable,
		httpSitesTable,
		enable2faTable,
		emailsTable,
	) = generateTables(credentials)

	string = StringIO()

	console = Console(file=string, color_system="truecolor")

	count = lambda x: f"[{'green' if len(x) < 2 else 'red'}]{len(x)-1}[/]"
	table = Table(title="\nSummary")
	table.add_column("Issue", style="cyan")
	table.add_column("No. Instances")
	table.add_row("Duplicate Passwords", count(duplicatePasswordsTable))
	table.add_row("Weak Passwords", count(weakPasswordsTable))
	table.add_row("HTTP Sites", count(httpSitesTable))
	table.add_row("Enable 2FA", count(enable2faTable))
	table.add_row("Emails", count(emailsTable))
	console.print(table)

	table = Table(title="\nDuplicate Passwords")
	table.add_column("Name", style="cyan")
	table.add_column("Username", style="magenta")
	table.add_column("Password", style="red")
	_ = [table.add_row(*x) for x in duplicatePasswordsTable[1:]]
	console.print(table)

	table = Table(title="\nWeak Passwords")
	table.add_column("Name", style="cyan")
	table.add_column("Username", style="magenta")
	table.add_column("Password", style="red")
	table.add_column("Score")
	table.add_column("Suggestion")
	scores = (
		"[black on red]Too Guessable[/]",
		"[black on yellow]Very Weak[/]",
		"[red]Weak[/]",
		"[yellow]Moderate[/]",
	)
	_ = [table.add_row(*x[:3], scores[int(x[3])], x[4]) for x in weakPasswordsTable[1:]]
	console.print(table)

	table = Table(title="\nHTTP Sites")
	table.add_column("Name", style="cyan")
	table.add_column("Username", style="magenta")
	_ = [table.add_row(*x) for x in httpSitesTable[1:]]
	console.print(table)

	table = Table(title="\nEnable 2FA")
	table.add_column("Name", style="cyan")
	table.add_column("Username", style="magenta")
	_ = [table.add_row(*x) for x in enable2faTable[1:]]
	console.print(table)

	table = Table(title="\nEmails")
	table.add_column("Emails", style="cyan")
	_ = [table.add_row(*x) for x in emailsTable[1:]]
	console.print(table)

	return string.getvalue()


def plainText(credentials: list[Credentials]) -> str:
	"""Format to plain text

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in plain text format
	"""
	return stripAnsi(ansi(credentials))


def markdown(credentials: list[Credentials]) -> str:
	"""Format to markdown

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in markdown format
	"""
	strBuf = []
	(
		duplicatePasswordsTable,
		weakPasswordsTable,
		httpSitesTable,
		enable2faTable,
		emailsTable,
	) = generateTables(credentials)
	count = lambda x: len(x) - 1
	toTB = lambda z: "\n".join([f'|{"|".join([str(y) for y in x])}|' for x in z[1:]])

	strBuf.append("\nSummary\n|Issue|No. Instances|\n|:--|:--|")
	strBuf.append(f"|duplicate_passwords|{count((duplicatePasswordsTable))}")
	strBuf.append(f"|weak_passwords|{count((weakPasswordsTable))}")
	strBuf.append(f"|http_sites|{count((httpSitesTable))}")
	strBuf.append(f"|enable_2fa|{count((enable2faTable))}")
	strBuf.append(f"|emails|{count((emailsTable))}")

	strBuf.append("\nDuplicate Passwords\n|Name|Username|Password|\n|:--|:--|:--|")
	strBuf.append(toTB(duplicatePasswordsTable))
	strBuf.append("\nWeak Passwords")
	strBuf.append("|Name|Username|Password|Score|Suggestion\n|:--|:--|:--|:--|:--|")
	strBuf.append(toTB(weakPasswordsTable))
	strBuf.append("\nHTTP Sites\n|Name|Username|\n|:--|:--")
	strBuf.append(toTB(httpSitesTable))
	strBuf.append("\nEnable 2FA\n|Name|Username|\n|:--|:--|")
	strBuf.append(toTB(enable2faTable))
	strBuf.append("\nEmails\n|Emails|\n|:--|")
	strBuf.append(toTB(emailsTable))

	return "\n".join(strBuf)


def jsonF(credentials: list[Credentials]) -> str:
	"""Format to json

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in json format
	"""
	(
		duplicatePasswordsTable,
		weakPasswordsTable,
		httpSitesTable,
		enable2faTable,
		emailsTable,
	) = generateTables(credentials)
	count = lambda x: len(x) - 1
	toDict = lambda z: [{z[0][y]: x[y] for y, _ in enumerate(x)} for x in z[1:]]

	data = {
		"summary": {
			"duplicate_passwords": count(duplicatePasswordsTable),
			"weak_passwords": count(weakPasswordsTable),
			"http_sites": count(httpSitesTable),
			"enable_2fa": count(enable2faTable),
			"emails": count(emailsTable),
		},
		"duplicate_passwords": toDict(duplicatePasswordsTable),
		"weak_passwords": toDict(weakPasswordsTable),
		"http_sites": toDict(httpSitesTable),
		"enable_2fa": toDict(enable2faTable),
		"emails": toDict(emailsTable),
	}

	return json.dumps({"info": INFO, "data": data}, indent="\t")


def raw(credentials: list[Credentials]) -> str:
	"""Format to raw json

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in raw json format
	"""
	return json.dumps({"info": INFO, "credentials": [x.__dict__ for x in credentials]}, indent="\t")


def rawCsv(credentials: list[Credentials]) -> str:
	"""Format to raw csv

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		str: string to send to specified output in raw csv format
	"""
	string = StringIO()
	data = [x.__dict__ for x in credentials]
	writer = csv.DictWriter(string, fieldnames=list(data[0]), lineterminator="\n")
	writer.writeheader()
	writer.writerows(data)
	return string.getvalue()



"""Credentials class and associated methods
"""
import re
from collections import Counter
from dataclasses import dataclass, field

from checkpasswords.auxiliary import (
	ZxcvbnScore,
	isHttp,
	isMfaAvailable,
	isMfaEnabled,
	passwordPrint,
	zxcvbnScore,
)


@dataclass
class Credentials:  # pylint: disable=too-many-instance-attributes
	"""Credentials storing raw data from IO and inferred data such as:
	- zxcvbnScore
	- isPasswordDuplicate
	- passwordPrint
	- isHttp
	- isMfaAvailable
	- isMfaEnabled

	Used to:
	- check for duplicate passwords
	- check for weak passwords
	- identify http sites
	- list available 2fa options using data from https://2fa.directory/
	"""

	name: str
	urls: list[str]
	username: str
	password: str
	notes: str
	totp: str
	zxcvbnScore: ZxcvbnScore = field(init=False)
	isPasswordDuplicate: bool = field(init=False)
	passwordPrint: str = field(init=False)
	isHttp: bool = field(init=False)
	isMfaAvailable: bool = field(init=False)
	isMfaEnabled: bool = field(init=False)

	def __post_init__(self):
		"""Populate/ update various attributes using auxiliary functions"""
		self.username = self.username.strip()
		urlstr = " ".join(self.urls).lower()
		self.zxcvbnScore = zxcvbnScore(self.password)
		self.isPasswordDuplicate = False
		self.passwordPrint = passwordPrint(self.password)
		self.isHttp = isHttp(urlstr)
		self.isMfaAvailable = isMfaAvailable(urlstr)
		self.isMfaEnabled = len(self.totp) > 0 or isMfaEnabled(self.notes)
		self.instructEnableMfa = self.isMfaAvailable and not self.isMfaEnabled


def applyPasswordDuplicate(credentials: list[Credentials]):
	"""Apply duplicate password flag to each credentials

	Args:
		credentials (list[Credentials]): list of all credentials
	"""
	duplicates = [
		k for k, v in Counter([x.password for x in credentials]).items() if v > 1 and len(k) > 0
	]
	for cred in credentials:
		cred.isPasswordDuplicate = cred.password in duplicates


def emails(credentials: list[Credentials]) -> set[str]:
	"""Return a set of unique emails from the list of credentials

	Args:
		credentials (list[Credentials]): list of all credentials

	Returns:
		set[str]: set of unique emails
	"""
	return {x.username.lower() for x in credentials if re.match(r"[^@]+@[^@]+\.[^@]+", x.username)}


def orderCredentials(credentials: list[Credentials]) -> list[Credentials]:
	"""Order credentials by password crack time

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV

	Returns:
		list[Credentials]: sorted credentials
	"""
	return sorted(credentials, key=lambda x: x.zxcvbnScore["crack_time"])


def generateTables(credentials: list[Credentials]) -> tuple[list[tuple[str, ...]], ...]:
	"""generateTables

	Args:
		credentials (list[Credentials]): list of credentials parsed from some input file.
		Such as a bitwarden export to CSV
	"""
	credentials = orderCredentials(credentials)
	applyPasswordDuplicate(credentials)

	duplicatePasswordsTable = [("Name", "Username", "Password")] + [  # type: ignore
		(cred.name, cred.username, cred.passwordPrint)
		for cred in credentials
		if cred.isPasswordDuplicate
	]

	weakPasswordsTable = [("Name", "Username", "Password", "Score", "Suggestion")] + [  # type: ignore
		(
			cred.name,
			cred.username,
			cred.passwordPrint,
			cred.zxcvbnScore["score"],
			cred.zxcvbnScore["suggestions"],
		)
		for cred in credentials
		if -1 < cred.zxcvbnScore["score"] < 4
	]

	httpSitesTable = [("Name", "Username")] + [  # type: ignore
		(cred.name, cred.username) for cred in credentials if cred.isHttp
	]

	enable2faTable = [("Name", "Username")] + [  # type: ignore
		(cred.name, cred.username) for cred in credentials if cred.instructEnableMfa
	]

	emailsTable = [("Emails",)] + [(email,) for email in sorted(emails(credentials))]  # type: ignore

	return duplicatePasswordsTable, weakPasswordsTable, httpSitesTable, enable2faTable, emailsTable



"""Use pass_import to convert an ambiguous password manager source file to a list[Credentials]
"""
import sys

from pass_import.__main__ import pass_import
from pass_import.auto import AutoDetect
from pass_import.tools import Config

from ..credentials import Credentials


def passImport(path: str, manager: str | None = None) -> list[Credentials]:
	"""Use pass_import to convert an ambiguous source file to a list[Credentials]

	Args:
		path (str): path to password source file
		manager (str, optional): specify a pasword manager if pass_import fails to identify it.
		Defaults to None.

	Returns:
		list[Credentials]: list of credentials used by the rest of checkpasswords
	"""
	# Use pass_import to convert an ambiguous source file to a list[Credentials]
	conf = Config()
	conf.readconfig(
		{
			"sroot": "",
		}
	)
	detect = AutoDetect(settings=conf.getsettings())
	passManager = detect.manager(path)
	if passManager is None:
		if manager:
			detect = AutoDetect(manager, settings=conf.getsettings())
			passManager = detect.format(path)
		else:
			print(" [x] Error: Must specify input_format e.g. -i bitwarden")
			sys.exit(1)
	conf["in"] = path
	conf["importer"] = passManager.name
	data = pass_import(conf, passManager)

	# Convert pass_import representation to checkpasswords representation (list[Credentials])
	credentials = []
	for row in data:
		credentials.append(
			Credentials(
				name=row["title"],
				urls=[
					x
					for x in ([row.get("url")] + [row.get(f"url{y}") for y in range(10)])
					if x is not None
				],
				username=row.get("login", ""),
				password=row.get("password", ""),
				notes=row.get("comments", ""),
				totp=row.get("otpauth", ""),
			)
		)
	return credentials



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['checkpasswords', 'checkpasswords.io']

package_data = \
{'': ['*']}

install_requires = \
['pass-import>=3.2,<4', 'rich>=12.2.0,<13', 'zxcvbn-python>=4.4.24,<5']

entry_points = \
{'console_scripts': ['CheckPasswords = checkpasswords:cli']}

setup_kwargs = {
    'name': 'checkpasswords',
    'version': '2022',
    'description': 'CheckPasswords uses pass_import to read a password manager source file to check for duplicate passwords; check for weak passwords; identify http sites; list available 2fa options; list emails',
    'long_description': '[![GitHub top language](https://img.shields.io/github/languages/top/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](../../)\n[![Repository size](https://img.shields.io/github/repo-size/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](../../)\n[![Issues](https://img.shields.io/github/issues/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](../../issues)\n[![License](https://img.shields.io/github/license/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](/LICENSE.md)\n[![Commit activity](https://img.shields.io/github/commit-activity/m/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](../../commits/master)\n[![Last commit](https://img.shields.io/github/last-commit/FHPythonUtils/CheckPasswords.svg?style=for-the-badge)](../../commits/master)\n[![PyPI Downloads](https://img.shields.io/pypi/dm/CheckPasswords.svg?style=for-the-badge)](https://pypistats.org/packages/CheckPasswords)\n[![PyPI Total Downloads](https://img.shields.io/badge/dynamic/json?style=for-the-badge&label=total%20downloads&query=%24.total_downloads&url=https%3A%2F%2Fapi.pepy.tech%2Fapi%2Fprojects%2Fcheckpasswords)](https://pepy.tech/project/CheckPasswords)\n[![PyPI Version](https://img.shields.io/pypi/v/CheckPasswords.svg?style=for-the-badge)](https://pypi.org/project/CheckPasswords)\n\n<!-- omit in toc -->\n# CheckPasswords\n\n<img src="readme-assets/icons/name.png" alt="Project Icon" width="750">\n\nUses pass_import to read a password manager source file storing raw data and infers data such as:\n\n- zxcvbnScore\n- isPasswordDuplicate\n- passwordPrint\n- isHttp\n- isMfaAvailable\n- isMfaEnabled\n\nUsed to:\n\n- check for duplicate passwords\n- check for weak passwords\n- identify http sites\n- list available 2fa options using data from https://2fa.directory/\n- list emails to submit to HIBP or similar\n\n<!-- omit in toc -->\n## Table of Contents\n\n- [Using](#using)\n\t- [CLI Help](#cli-help)\n\t- [Example Output](#example-output)\n- [Documentation](#documentation)\n- [Install With PIP](#install-with-pip)\n- [Language information](#language-information)\n\t- [Built for](#built-for)\n- [Install Python on Windows](#install-python-on-windows)\n\t- [Chocolatey](#chocolatey)\n\t- [Windows - Python.org](#windows---pythonorg)\n- [Install Python on Linux](#install-python-on-linux)\n\t- [Apt](#apt)\n\t- [Dnf](#dnf)\n- [Install Python on MacOS](#install-python-on-macos)\n\t- [Homebrew](#homebrew)\n\t- [MacOS - Python.org](#macos---pythonorg)\n- [How to run](#how-to-run)\n\t- [Windows](#windows)\n\t- [Linux/ MacOS](#linux-macos)\n- [Download Project](#download-project)\n\t- [Clone](#clone)\n\t\t- [Using The Command Line](#using-the-command-line)\n\t\t- [Using GitHub Desktop](#using-github-desktop)\n\t- [Download Zip File](#download-zip-file)\n- [Community Files](#community-files)\n\t- [Licence](#licence)\n\t- [Changelog](#changelog)\n\t- [Code of Conduct](#code-of-conduct)\n\t- [Contributing](#contributing)\n\t- [Security](#security)\n\t- [Support](#support)\n\t- [Rationale](#rationale)\n\n## Using\n\n### CLI Help\n\n```txt\nusage: __main__.py [-h] [--output-format OUTPUT_FORMAT] [--input-format INPUT_FORMAT] [--file FILE] [--no-colour]\n                   credentials\n\ncheckpasswords: Uses pass_import to read a password manager source file storing raw\ndata and infers data such as:\n\n- zxcvbnScore\n- isPasswordDuplicate\n- passwordPrint\n- isHttp\n- isMfaAvailable\n- isMfaEnabled\n\nUsed to:\n\n- check for duplicate passwords\n- check for weak passwords\n- identify http sites\n- list available 2fa options using data from https://2fa.directory/\n- list emails to submit to HIBP or similar\n\npositional arguments:\n  credentials           Credentials/ passwords file to check\n\noptions:\n  -h, --help            show this help message and exit\n  --output-format OUTPUT_FORMAT, -o OUTPUT_FORMAT\n                        Output format. One of [\'ansi\', \'plain\', \'markdown\', \'json\', \'raw\', \'raw-csv\']. default=ansi\n  --input-format INPUT_FORMAT, -i INPUT_FORMAT\n                        Input format. One of [\'1password\', \'aegis\', \'andotp\', \'apple-keychain\', \'bitwarden\', \'blur\', \'buttercup\', \'chrome\', \'clipperz\', \'csv\', \'dashlane\', \'encryptr\', \'enpass\', \'firefox\', \'fpm\', \'freeotp+\', \'gnome\', \'gnome-auth\', \'gopass\', \'gorilla\', \'kedpm\', \'keepass\', \'keepassx\', \'keepassx2\', \'keepassxc\', \'keeper\', \'lastpass\', \'myki\', \'network-manager\', \'padlock\', \'pass\', \'passman\', \'passpack\', \'passpie\', \'pwsafe\', \'revelation\', \'roboform\', \'saferpass\', \'upm\', \'zoho\']\n  --file FILE, -f FILE  Filename to write to (omit for stdout)\n  --no-colour, -z       No ANSI colours\n```\n\n### Example Output\n\n```txt\n\n                Summary\n┏━━━━━━━━━━━━━━━━━━━━━┳━━━━━━━━━━━━━━━┓\n┃ Issue               ┃ No. Instances ┃\n┡━━━━━━━━━━━━━━━━━━━━━╇━━━━━━━━━━━━━━━┩\n│ Duplicate Passwords │ 2             │\n│ Weak Passwords      │ 9             │\n│ HTTP Sites          │ 9             │\n│ Enable 2FA          │ 16            │\n│ Emails              │ 17            │\n└─────────────────────┴───────────────┘\n\n...\n```\n\n## Documentation\n\nA high-level overview of how the documentation is organized organized will help you know\nwhere to look for certain things:\n\n<!--\n- [Tutorials](/documentation/tutorials) take you by the hand through a series of steps to get\n  started using the software. Start here if you’re new.\n-->\n- The [Technical Reference](/documentation/reference) documents APIs and other aspects of the\n  machinery. This documentation describes how to use the classes and functions at a lower level\n  and assume that you have a good high-level understanding of the software.\n<!--\n- The [Help](/documentation/help) guide provides a starting point and outlines common issues that you\n  may have.\n-->\n\n## Install With PIP\n\n```python\npip install CheckPasswords\n```\n\nHead to https://pypi.org/project/CheckPasswords/ for more info\n\n## Language information\n\n### Built for\n\nThis program has been written for Python versions 3.7 - 3.10 and has been tested with both 3.7 and\n3.10\n\n## Install Python on Windows\n\n### Chocolatey\n\n```powershell\nchoco install python\n```\n\n### Windows - Python.org\n\nTo install Python, go to https://www.python.org/downloads/windows/ and download the latest\nversion.\n\n## Install Python on Linux\n\n### Apt\n\n```bash\nsudo apt install python3.x\n```\n\n### Dnf\n\n```bash\nsudo dnf install python3.x\n```\n\n## Install Python on MacOS\n\n### Homebrew\n\n```bash\nbrew install python@3.x\n```\n\n### MacOS - Python.org\n\nTo install Python, go to https://www.python.org/downloads/macos/ and download the latest\nversion.\n\n## How to run\n\n### Windows\n\n- Module\n\t`py -3.x -m [module]` or `[module]` (if module installs a script)\n\n- File\n\t`py -3.x [file]` or `./[file]`\n\n### Linux/ MacOS\n\n- Module\n\t`python3.x -m [module]` or `[module]` (if module installs a script)\n\n- File\n\t`python3.x [file]` or `./[file]`\n\n## Download Project\n\n### Clone\n\n#### Using The Command Line\n\n1. Press the Clone or download button in the top right\n2. Copy the URL (link)\n3. Open the command line and change directory to where you wish to\nclone to\n4. Type \'git clone\' followed by URL in step 2\n\t```bash\n\tgit clone https://github.com/FHPythonUtils/CheckPasswords\n\t```\n\nMore information can be found at\nhttps://help.github.com/en/articles/cloning-a-repository\n\n#### Using GitHub Desktop\n\n1. Press the Clone or download button in the top right\n2. Click open in desktop\n3. Choose the path for where you want and click Clone\n\nMore information can be found at\nhttps://help.github.com/en/desktop/contributing-to-projects/cloning-a-repository-from-github-to-github-desktop\n\n### Download Zip File\n\n1. Download this GitHub repository\n2. Extract the zip archive\n3. Copy/ move to the desired location\n\n## Community Files\n\n### Licence\n\nMIT License\nCopyright (c) FredHappyface\n(See the [LICENSE](/LICENSE.md) for more information.)\n\n### Changelog\n\nSee the [Changelog](/CHANGELOG.md) for more information.\n\n### Code of Conduct\n\nOnline communities include people from many backgrounds. The *Project*\ncontributors are committed to providing a friendly, safe and welcoming\nenvironment for all. Please see the\n[Code of Conduct](https://github.com/FHPythonUtils/.github/blob/master/CODE_OF_CONDUCT.md)\n for more information.\n\n### Contributing\n\nContributions are welcome, please see the\n[Contributing Guidelines](https://github.com/FHPythonUtils/.github/blob/master/CONTRIBUTING.md)\nfor more information.\n\n### Security\n\nThank you for improving the security of the project, please see the\n[Security Policy](https://github.com/FHPythonUtils/.github/blob/master/SECURITY.md)\nfor more information.\n\n### Support\n\nThank you for using this project, I hope it is of use to you. Please be aware that\nthose involved with the project often do so for fun along with other commitments\n(such as work, family, etc). Please see the\n[Support Policy](https://github.com/FHPythonUtils/.github/blob/master/SUPPORT.md)\nfor more information.\n\n### Rationale\n\nThe rationale acts as a guide to various processes regarding projects such as\nthe versioning scheme and the programming styles used. Please see the\n[Rationale](https://github.com/FHPythonUtils/.github/blob/master/RATIONALE.md)\nfor more information.\n',
    'author': 'FredHappyface',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FHPythonUtils/CheckPasswords',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
